@@ -12,7 +12,7 @@ use Tickit::Widget::HBox;
 
 my $tickit = Tickit->new();
 
-my @basekeys = qw( a i Space Tab Enter Up Escape );
+my @basekeys = qw( a i Space Tab Enter Up Insert );
 
 my $vbox = Tickit::Widget::VBox->new( spacing => 1 );
 my $hbox;
@@ -36,23 +36,33 @@ foreach my $mbits ( 0 .. 7 ) {
    foreach ( @basekeys ) {
       my $basekey = $_; # avoid alias
 
-      my $static = Tickit::Widget::Static->new( text => "--    " );
+      my $static;
 
-      my $thismod = $modifier;
-      # Keybindings are weirder
-      if( length( $basekey ) == 1 ) {
-         $thismod =~ s/S-// and $basekey = uc $basekey;
-      }
-      elsif( $basekey eq "Space" ) {
-         $basekey = " ";
+      if( $modifier =~ m/S-/ && $basekey eq "Insert" ) {
+         $static = Tickit::Widget::Static->new(
+            text => "XX    ",
+            fg   => "red",
+         );
       }
+      else {
+         $static = Tickit::Widget::Static->new( text => "--    " );
 
-      $tickit->bind_key(
-         "$thismod$basekey" => sub {
-            $static->pen->chattr( fg => "green" );
-            $static->set_text( "OK    " );
+         my $thismod = $modifier;
+         # Keybindings are weirder
+         if( length( $basekey ) == 1 ) {
+            $thismod =~ s/S-// and $basekey = uc $basekey;
          }
-      );
+         elsif( $basekey eq "Space" ) {
+            $basekey = " ";
+         }
+
+         $tickit->bind_key(
+            "$thismod$basekey" => sub {
+               $static->pen->chattr( fg => "green" );
+               $static->set_text( "OK    " );
+            }
+         );
+      }
 
       $hbox->add( $static );
    }
