@@ -8,10 +8,15 @@ package Tickit::Term;
 use strict;
 use warnings;
 
-our $VERSION = '0.13';
+our $VERSION = '0.14';
 
 use Encode qw( find_encoding );
+use Term::Size;
+use Term::TermKey 0.11 qw( FORMAT_ALTISMETA FLAG_UTF8 FLAG_RAW FLAG_EINTR RES_KEY RES_AGAIN );
 use Term::Terminfo;
+use Time::HiRes qw( time );
+
+use constant FLAGS_UTF8_RAW => FLAG_UTF8|FLAG_RAW;
 
 use Tickit::Pen;
 
@@ -88,6 +93,43 @@ a string of bytes.
 
 Such an interface is supported by an C<IO::Handle> object.
 
+=item output_handle => HANDLE
+
+Optional. If supplied, will be used as the terminal filehandle for querying
+the size. Even if supplied, all writing operations will use the C<writer>
+function rather than performing IO operations on this filehandle.
+
+=item input_handle => HANDLE
+
+Optional. If supplied, will be used as the terminal filehandle for reading
+keypress and other events.
+
+=item on_resize => CODE
+
+Optional. Event handler function for when the terminal window is resized. Will
+be passed the C<Tickit::Term> instance, and the new size.
+
+ $on_resize->( $term, $lines, $cols )
+
+=item on_key => CODE
+
+Optional. Event handler function for when a key is pressed. Will be passed the
+C<Tickit::Term> instance, a type string (either C<text> for unmodified Unicode
+or C<key> for special keys or modified Unicode), a string containing a
+representation of the text or key, and the underlying C<Term::TermKey::Key>
+event.
+
+ $on_key->( $term, $type, $str, $key )
+
+=item on_mouse => CODE
+
+Optional. Event handler function for when a mouse button is pressed or
+released, or the cursor dragged with a button pressed. Will be passed the
+C<Tickit::Term> instance, a string indicating C<press>, C<drag> or C<release>,
+the button number, and the 0-based line and column index.
+
+ $on_mouse->( $term, $ev, $button, $line, $col )
+
 =back
 
 =cut
@@ -99,8 +141,24 @@ sub new
 
    my $encoding = delete $params{encoding};
 
+   my $termkey = Term::TermKey->new( $params{input_handle}, FLAG_EINTR );
+
+   my $is_utf8 = defined $params{UTF8} ? $params{UTF8} : ${^UTF8LOCALE};
+   $termkey->set_flags( ( $termkey->get_flags & ~FLAGS_UTF8_RAW ) |
+                        ( $is_utf8 ? FLAG_UTF8 : FLAG_RAW ) );
+
+   $encoding = "UTF-8" if $is_utf8;
+
    my $self = bless {
       writer => $params{writer},
+
+      input_handle  => $params{input_handle},
+      output_handle => $params{output_handle},
+
+      lines => 0, # Will be filled in by refresh_size
+      cols  => 0,
+
+      termkey => $termkey,
    }, $class;
 
    my $ti = Term::Terminfo->new();
@@ -108,6 +166,15 @@ sub new
    # Precache some terminfo flags that we know won't change
    $self->{has_bce} = $ti->getflag( "bce" );
 
+   $self->{lines} = $ti->getnum( "lines" );
+   $self->{cols}  = $ti->getnum( "cols" );
+
+   $self->refresh_size if $self->{output_handle};
+
+   $self->set_on_resize( $params{on_resize} ) if $params{on_resize};
+   $self->set_on_key   ( $params{on_key}    ) if $params{on_key};
+   $self->set_on_mouse ( $params{on_mouse}  ) if $params{on_mouse};
+
    if( defined $encoding ) {
       $self->{encoder} = find_encoding( $encoding );
    }
@@ -130,12 +197,108 @@ sub write
 
 =cut
 
+=head2 $fh = $term->get_input_handle
+
+Returns the input handle set by the C<input_handle> constructor arg.
+
+=cut
+
+sub get_input_handle
+{
+   my $self = shift;
+   return $self->{input_handle};
+}
+
+=head2 $fh = $term->get_output_handle
+
+Returns the output handle set by the C<output_handle> constructor arg.
+
+=cut
+
+sub get_output_handle
+{
+   my $self = shift;
+   return $self->{output_handle};
+}
+
+=head2 $term->set_on_resize( $on_resize )
+
+=cut
+
+sub set_on_resize
+{
+   my $self = shift;
+   ( $self->{on_resize} ) = @_;
+}
+
+=head2 $term->set_on_key( $on_key )
+
+=cut
+
+sub set_on_key
+{
+   my $self = shift;
+   ( $self->{on_key} ) = @_;
+}
+
+=head2 $term->set_on_mouse( $on_mouse )
+
+Set a new CODE references to handle events.
+
+=cut
+
+sub set_on_mouse
+{
+   my $self = shift;
+   ( $self->{on_mouse} ) = @_;
+}
+
+=head2 $term->refresh_size
+
+If a filehandle was supplied to the constructor, fetch the size of the
+terminal and update the cached sizes in the object. May invoke C<on_resize> if
+the new size is different.
+
+=cut
+
+sub refresh_size
+{
+   my $self = shift;
+   return unless my $fh = $self->get_output_handle;
+   my ( $cols, $lines ) = Term::Size::chars $fh;
+   # Due to Term::Size bug RT76292, $cols is always defined
+   $self->set_size( $lines, $cols ) if defined $lines;
+}
+
+=head2 $term->set_size( $lines, $cols )
+
+Defines the size of the terminal. Invoke C<on_resize> if the new size is
+different.
+
+=cut
+
 sub set_size
 {
    my $self = shift;
-   ( $self->{lines}, $self->{cols} ) = @_;
+   my ( $lines, $cols ) = @_;
+
+   if( $lines != $self->{lines} or $cols != $self->{cols} ) {
+      $self->{lines} = $lines;
+      $self->{cols}  = $cols;
+
+      $self->{on_resize}->( $self, $lines, $cols ) if $self->{on_resize};
+   }
 }
 
+=head2 $lines = $term->lines
+
+=head2 $cols = $term->cols
+
+Query the size of the terminal, as set by the most recent C<refresh_size> or
+C<set_size> operation.
+
+=cut
+
 sub lines { shift->{lines} }
 sub cols  { shift->{cols}  }
 
@@ -394,12 +557,6 @@ sub erasech
    }
 }
 
-=head2 $term->insertch( $count )
-
-Insert C<$count> blank characters, shifting following text to the right.
-
-=cut
-
 sub insertch
 {
    my $self = shift;
@@ -408,13 +565,6 @@ sub insertch
    $self->write( sprintf "${CSI}%d@", $count );
 }
 
-=head2 $term->deletech( $count )
-
-Delete the following C<$count> characters, shifting the remaining text to the
-left. The terminal will fill the empty region with blanks.
-
-=cut
-
 sub deletech
 {
    my $self = shift;
@@ -468,6 +618,141 @@ sub mode_mouse
    $self->write( $on ? "${CSI}?1002h" : "${CSI}?1002l" );
 }
 
+=head2 $term->input_push_bytes( $bytes )
+
+Feeds more bytes of input. May result in C<on_key> or C<on_mouse> events.
+
+=cut
+
+sub input_push_bytes
+{
+   my $self = shift;
+   my ( $bytes ) = @_;
+
+   my $termkey = $self->{termkey};
+
+   $termkey->push_bytes( $bytes );
+
+   $self->_get_keys;
+}
+
+=head2 $term->input_readable
+
+Informs the term that the input handle may be readable. Attempts to read more
+bytes of input. May result in C<on_key> or C<on_mouse> events.
+
+=cut
+
+sub input_readable
+{
+   my $self = shift;
+
+   my $termkey = $self->{termkey};
+
+   $termkey->advisereadable;
+
+   $self->_get_keys;
+}
+
+=head2 $term->input_wait
+
+Block until some input is available, and process it. Returns after one round
+of input has been processed. May result in C<on_key> or C<on_mouse> events.
+
+=cut
+
+sub input_wait
+{
+   my $self = shift;
+
+   my $termkey = $self->{termkey};
+
+   my $key;
+   $termkey->waitkey( $key );
+
+   $self->_got_key( $key );
+
+   # Might as well process the remaining if there are any
+   while( $termkey->getkey( $key ) == RES_KEY ) {
+      $self->_got_key( $key );
+   }
+}
+
+sub _got_key
+{
+   my $self = shift;
+   my ( $key ) = @_;
+
+   my $termkey = $self->{termkey};
+
+   if( $key->type_is_unicode and !$key->modifiers ) {
+      $self->{on_key}->( $self, text => $key->utf8, $key ) if $self->{on_key};
+   }
+   elsif( $key->type_is_mouse ) {
+      my ( $ev, $button, $line, $col ) = $termkey->interpret_mouse( $key );
+      my $evname = (qw( * press drag release ))[$ev];
+      $self->{on_mouse}->( $self, $evname, $button, $line - 1, $col - 1 ) if $self->{on_mouse};
+   }
+   else {
+      $self->{on_key}->( $self, key => $termkey->format_key( $key, FORMAT_ALTISMETA ), $key ) if $self->{on_key};
+   }
+}
+
+sub _get_keys
+{
+   my $self = shift;
+
+   my $termkey = $self->{termkey};
+
+   my $res;
+   while( ( $res = $termkey->getkey( my $key ) ) == RES_KEY ) {
+      $self->_got_key( $key );
+   }
+
+   if( $res == RES_AGAIN ) {
+      $self->{input_timeout_at} = time + ( $termkey->get_waittime / 1000 ); # msec
+   }
+   else {
+      undef $self->{input_timeout_at};
+   }
+}
+
+sub _force_key
+{
+   my $self = shift;
+
+   my $termkey = $self->{termkey};
+
+   if( $termkey->getkey_force( my $key ) == RES_KEY ) {
+      $self->_got_key( $key );
+   }
+}
+
+=head2 $timeout = $term->check_timeout
+
+Returns a number in seconds to represent when the next timeout should occur on
+the terminal, or C<undef> if nothing is waiting. May invoke expired timeouts,
+and cause a C<on_key> event to occur.
+
+=cut
+
+sub check_timeout
+{
+   my $self = shift;
+   return undef unless defined( my $timeout_at = $self->{input_timeout_at} );
+
+   my $timeout = $timeout_at - time;
+
+   if( $timeout <= 0 ) {
+      $self->_force_key;
+      undef $self->{input_timeout_at};
+      return undef;
+   }
+   else {
+      return $timeout;
+   }
+}
+
 =head1 TODO
 
 =over 4
